@layer wf{.WdsTitle2[data-v-84903d5f]{padding-bottom:8px}.BuilderSettingsArtifactAPITriggerDetails[data-v-84903d5f]{display:flex;flex-direction:column;gap:8px;padding:24px}.BuilderSettingsArtifactAPITriggerDetails__tabs[data-v-84903d5f]{gap:10px}.BuilderSettingsArtifactAPITriggerDetails__builder[data-v-84903d5f],.BuilderSettingsArtifactAPITriggerDetails__api[data-v-84903d5f]{display:flex;flex-direction:column;gap:8px;margin-top:8px}.BuilderSettingsArtifactAPITriggerDetails__steps[data-v-84903d5f]{display:flex;flex-direction:column;gap:8px;list-style-type:decimal;padding-left:16px}.BuilderSettingsArtifactAPITriggerDetails__step:first-of-type>.BuilderSettingsArtifactAPITriggerDetails__step__hint[data-v-84903d5f]{margin-top:0}.BuilderSettingsArtifactAPITriggerDetails__step__hint[data-v-84903d5f]{margin:12px 0}}@layer wf{.BuilderSettingsArtifactAPITriggerDetails__tabs>.WdsTab{font-size:12px;padding:2px 8px}}
