@layer wf{.BuilderSidebarComponentTreeBranch[data-v-a553e087]{min-width:170px}.BuilderSidebarComponentTreeBranch__nameRight[data-v-a553e087]{display:flex;align-items:center;gap:4px;flex-wrap:nowrap;text-wrap:nowrap;text-overflow:ellipsis;min-width:0}.previewText[data-v-a553e087]{flex-wrap:nowrap;text-wrap:nowrap;flex-grow:1;white-space:nowrap;text-overflow:ellipsis;overflow:hidden}.middot[data-v-a553e087]{display:inline-block;border-radius:3px;width:3px;min-width:3px;height:3px;background:var(--builderSecondaryTextColor);opacity:.5}}@layer wf{.BuilderSidebarComponentTree[data-v-45d321fc]{height:100%;position:relative}.category .header[data-v-45d321fc]{font-size:12px;font-weight:500;line-height:12px;letter-spacing:1.3px;text-transform:uppercase;color:var(--builderSecondaryTextColor);margin-bottom:16px}.rootBranch[data-v-45d321fc]{flex:1 0 auto}.tools[data-v-45d321fc]{display:grid;grid-template-columns:24px 1fr;grid-template-rows:auto;padding:0 8px;row-gap:12px;column-gap:4px}.add[data-v-45d321fc]{flex:0 0 var(--builderPanelSwitcherHeight);bottom:0;height:var(--builderPanelSwitcherHeight);display:flex;align-items:center;justify-content:center;border-top:1px solid var(--builderSeparatorColor);background:var(--builderBackgroundColor)}.sections[data-v-45d321fc]{display:flex;flex-direction:column;gap:16px}.section[data-v-45d321fc]{display:flex;flex-direction:column}.section__header[data-v-45d321fc]{display:flex;align-items:center;justify-content:space-between;padding:8px 16px;border-bottom:1px solid var(--builderSeparatorColor)}.section__title[data-v-45d321fc]{font-size:11px;font-weight:600;text-transform:uppercase;letter-spacing:.5px;color:var(--builderSecondaryTextColor)}.section__content[data-v-45d321fc]{padding:8px 0}.section__empty[data-v-45d321fc]{padding:8px 16px;font-size:12px;color:var(--builderSecondaryTextColor);font-style:italic}}
