@layer wf{.DeploySharedBlueprint[data-v-1ad841f6]{display:flex;flex-direction:column;gap:16px;padding:16px}.DeploySharedBlueprint__textarea[data-v-1ad841f6]{width:100%;min-height:100px;padding:8px;border:1px solid var(--builderSeparatorColor);border-radius:4px;font-size:14px;font-family:inherit;resize:vertical}.DeploySharedBlueprint__textarea[data-v-1ad841f6]:focus{outline:none;border-color:var(--builderPrimaryColor)}.DeploySharedBlueprint__textarea--error[data-v-1ad841f6]{border-color:var(--builderErrorColor)}.DeploySharedBlueprint__globalOption[data-v-1ad841f6]{display:flex;flex-direction:column;gap:4px;padding:12px;background:var(--builderSubtleSeparatorColor);border-radius:4px}.DeploySharedBlueprint__checkbox[data-v-1ad841f6]{display:flex;align-items:center;gap:8px;cursor:pointer;font-size:14px}.DeploySharedBlueprint__checkbox input[data-v-1ad841f6]{cursor:pointer}.DeploySharedBlueprint__hint[data-v-1ad841f6]{font-size:12px;color:var(--builderSecondaryTextColor);margin:0}.DeploySharedBlueprint__success[data-v-1ad841f6]{display:flex;flex-direction:column;gap:8px;padding:12px;background:var(--builderSuccessBackgroundColor, #e6f7e6);border:1px solid var(--builderSuccessColor, #4caf50);border-radius:4px}.DeploySharedBlueprint__success p[data-v-1ad841f6]{margin:0;font-weight:500}.DeploySharedBlueprint__success a[data-v-1ad841f6]{color:var(--builderPrimaryColor);word-break:break-all}}
