@layer wf{.WdsTab[data-v-db73c62c]{position:relative;border:none;background-color:transparent;cursor:pointer}.WdsTab__indicator[data-v-db73c62c]{pointer-events:none;position:absolute;bottom:0;left:0;right:0;height:2px;width:100%;background:currentColor;opacity:0}.WdsTab--disabled[data-v-db73c62c]{cursor:not-allowed}.WdsTab--variant-bar[data-v-db73c62c]{padding:10px 0;font-size:14px;line-height:180%;color:var(--wdsColorGray5)}.WdsTab--variant-bar.WdsTab--selected[data-v-db73c62c]{color:var(--wdsColorBlack)}.WdsTab--variant-bar.WdsTab--selected .WdsTab__indicator[data-v-db73c62c]{opacity:1}.WdsTab--variant-chip[data-v-db73c62c]{padding:4px 20px;border-radius:8px}.WdsTab--variant-chip[data-v-db73c62c]:hover{background-color:var(--wdsColorBlue1)}.WdsTab--variant-chip.WdsTab--selected[data-v-db73c62c]{background-color:var(--wdsColorBlue2)}.WdsTab--variant-chip.WdsTab--disabled[data-v-db73c62c]{color:var(--wdsColorGray4)}}@layer wf{.WdsTabs[data-v-6e6e5572]{display:flex}.WdsTabs--variant-bar[data-v-6e6e5572]{gap:32px;overflow-x:auto;border-bottom:1px solid var(--wdsColorGray2)}.WdsTabs--variant-chip[data-v-6e6e5572]{gap:16px;flex-wrap:wrap}}
