import{d as b,an as f,aW as k,ap as m,o as s,f as d,g as i,x,y as C,t as c,S as n,R as r,aq as u,G as _,_ as v}from"./index-dyx4v2bZ.js";const y=["for"],W={class:"WdsCheckbox__checkbox"},p={key:0,class:"WdsCheckbox__label"},g={key:1,class:"WdsCheckbox__detail"},B=["id","checked","disabled","aria-invalid"],V=b({__name:"WdsCheckbox",props:f({label:{type:String,required:!1,default:void 0},detail:{type:String,required:!1,default:void 0},disabled:{type:Boolean,required:!1},invalid:{type:Boolean,default:!1}},{modelValue:{type:Boolean,default:!1},modelModifiers:{}}),emits:["update:modelValue"],setup(e){const l=k(),a=m(e,"modelValue");function h(o){a.value=o.target.checked}return(o,t)=>(s(),d("label",{for:r(l),class:_(["WdsCheckbox",{"WdsCheckbox--disabled":e.disabled,"WdsCheckbox--checked":a.value}]),onMousedown:t[0]||(t[0]=u(()=>{},["prevent"]))},[i("div",W,[x(C,{name:"check"})]),e.label?(s(),d("span",p,c(e.label),1)):n("",!0),e.detail?(s(),d("span",g,c(e.detail),1)):n("",!0),i("input",{id:r(l),type:"checkbox",checked:a.value,disabled:e.disabled,"aria-invalid":e.invalid,onChange:u(h,["stop"])},null,40,B)],42,y))}}),q=v(V,[["__scopeId","data-v-56650334"]]);export{q as W};
