@layer wf{.BuilderTemplateEditor[data-v-9c819264] .tiptap:focus-visible{outline:none}.BuilderTemplateEditor[data-v-9c819264] .BuilderTemplateEditorInterpolation{background-color:var(--wdsColorGray1);color:var(--wdsColorBlack);padding:2px 8px;border-radius:4px}}
