@layer wf{.BuilderVault[data-v-4766db1d]{max-width:500px;margin-left:auto;margin-right:auto;padding:40px;display:flex;flex-direction:column;gap:24px}.BuilderVault h1[data-v-4766db1d]{display:flex;gap:12px;align-items:center;font-weight:500;font-size:20px}.BuilderVault__editor__form[data-v-4766db1d]{display:grid;grid-template-columns:1fr 1fr auto;align-items:flex-start;gap:10px;margin-bottom:22px}.BuilderVault__editor__form__loader[data-v-4766db1d]{height:40px}.BuilderVault__editor__toolbar[data-v-4766db1d]{display:flex;gap:8px;justify-content:space-between}.BuilderVault__editor__form__deleteBtn[data-v-4766db1d]{margin-top:4px}.BuilderVault__editor__form__labelKey[data-v-4766db1d]{grid-column-start:1}.BuilderVault__editor__form__labelValue[data-v-4766db1d]{grid-column-start:2;grid-column-end:-1}}
