@layer wf{.component[data-v-15f0885c],.colorTransformer[data-v-15f0885c]{--softenedAccentColor: var(--accentColor);--intensifiedAccentColor: var(--accentColor);--softenedButtonColor: var(--wdsColorBlue6);--intensifiedButtonColor: var(--buttonColor);--softenedSeparatorColor: var(--separatorColor);--intensifiedSeparatorColor: var(--separatorColor)}@supports (color: hsl(from red h s calc(l - 20))){.component[data-v-15f0885c],.colorTransformer[data-v-15f0885c]{--softenedAccentColor: hsl( from var(--accentColor) calc(h - 12) calc(s + 0) calc(l + .21) );--intensifiedAccentColor: hsl( from var(--accentColor) calc(h + 1) calc(s - .33) calc(l - .1) );--softenedButtonColor: hsl( from var(--buttonColor, var(--wdsColorBlue6)) calc(h - 12) calc(s + 0) calc(l + .21) );--intensifiedButtonColor: hsl( from var(--buttonColor) calc(h + 1) calc(s - 33) calc(l - 10) );--softenedSeparatorColor: hsl( from var(--separatorColor) calc(h + -0) calc(s + .05) calc(l + .06) );--intensifiedSeparatorColor: hsl( from var(--separatorColor) calc(h + -0) calc(s - .06) calc(l - .52) )}}.WdsFieldWrapper[data-v-15f0885c]{display:flex;flex-direction:column;gap:4px}.WdsFieldWrapper__title[data-v-15f0885c]{color:var(--primaryTextColor);display:grid;grid-template-columns:1fr auto;align-items:center;flex-wrap:wrap;gap:8px}.WdsFieldWrapper__title__label[data-v-15f0885c]{font-size:14px;font-weight:400;line-height:20px}.WdsFieldWrapper__title__label__unit[data-v-15f0885c]{color:var(--secondaryTextColor)}.WdsFieldWrapper__hint[data-v-15f0885c],.WdsFieldWrapper__error[data-v-15f0885c]{margin-top:4px;font-family:Poppins;font-size:12px;font-weight:400;line-height:160%}.WdsFieldWrapper__error[data-v-15f0885c]:first-letter{text-transform:capitalize}.WdsFieldWrapper__hint[data-v-15f0885c]{color:var(--secondaryTextColor)}.WdsFieldWrapper__error[data-v-15f0885c]{color:var(--builderErrorColor)}.temporaryMissingBar[data-v-15f0885c]{background:var(--separatorColor);color:var(--secondaryTextColor);padding:16px;border-radius:8px}}
