@layer wf{.WdsButtonSplit[data-v-e9224293]{position:relative;display:flex;border-radius:300px;box-shadow:var(--buttonShadow);border:1px solid transparent;align-items:center;height:40px;gap:8px;padding-left:12px;padding-right:12px}.WdsButtonSplit--disabled[data-v-e9224293]{opacity:40%}.WdsButtonSplit--disabled .WdsButtonSplit__dropdownTrigger[data-v-e9224293],.WdsButtonSplit--disabled .WdsButtonSplit__main[data-v-e9224293]{cursor:not-allowed}.WdsButtonSplit__dropdownTrigger[data-v-e9224293],.WdsButtonSplit__main[data-v-e9224293]{background-color:transparent;border:none;cursor:pointer}.WdsButtonSplit__main[data-v-e9224293]{display:flex;align-items:center;justify-content:flex-start;gap:8px;font-weight:600}.WdsButtonSplit__dropdownTrigger[data-v-e9224293]{display:flex;align-items:center;justify-content:center;font-size:20px;border-radius:50%;height:20px;width:20px}.WdsButtonSplit__divider[data-v-e9224293]{display:block;width:1px;height:100%;border:none;height:24px}.WdsButtonSplit--variant-primary .WdsButtonSplit__divider[data-v-e9224293]{background:var(--wdsColorBlue4)}.WdsButtonSplit--variant-primary[data-v-e9224293],.WdsButtonSplit--variant-primary .WdsButtonSplit__dropdownTrigger[data-v-e9224293]{color:var(--wdsColorWhite);background:var(--wdsColorBlue5)}.WdsButtonSplit--variant-primary[data-v-e9224293]:not(.WdsButtonSplit--disabled):hover,.WdsButtonSplit--variant-primary:not(.WdsButtonSplit--disabled) .WdsButtonSplit__dropdownTrigger[data-v-e9224293]:hover{background:var(--wdsColorBlue6)}.WdsButtonSplit--variant-secondary .WdsButtonSplit__divider[data-v-e9224293]{background:var(--wdsColorGray5)}.WdsButtonSplit--variant-secondary[data-v-e9224293],.WdsButtonSplit--variant-secondary .WdsButtonSplit__dropdownTrigger[data-v-e9224293]{color:var(--wdsColorWhite);background:var(--wdsColorBlack)}.WdsButtonSplit--variant-secondary[data-v-e9224293]:not(.WdsButtonSplit--disabled):hover,.WdsButtonSplit--variant-secondary:not(.WdsButtonSplit--disabled) .WdsButtonSplit__dropdownTrigger[data-v-e9224293]:hover{background:var(--wdsColorGray6)}.WdsButtonSplit--variant-tertiary .WdsButtonSplit__divider[data-v-e9224293]{background:var(--wdsColorGray2)}.WdsButtonSplit--variant-tertiary[data-v-e9224293],.WdsButtonSplit--variant-tertiary .WdsButtonSplit__dropdownTrigger[data-v-e9224293]{color:var(--wdsColorBlack);background:var(--wdsColorWhite)}.WdsButtonSplit--variant-tertiary[data-v-e9224293]:not(.WdsButtonSplit--disabled):hover,.WdsButtonSplit--variant-tertiary:not(.WdsButtonSplit--disabled) .WdsButtonSplit__dropdownTrigger[data-v-e9224293]:hover{color:var(--wdsColorGray4)}.WdsButtonSplit--variant-special .WdsButtonSplit__divider[data-v-e9224293]{background:var(--wdsColorBlue5)}.WdsButtonSplit--variant-special[data-v-e9224293],.WdsButtonSplit--variant-special .WdsButtonSplit__dropdownTrigger[data-v-e9224293]{color:var(--wdsColorBlue5);background:var(--wdsColorBlue2)}.WdsButtonSplit--variant-special[data-v-e9224293]:not(.WdsButtonSplit--disabled):hover,.WdsButtonSplit--variant-special:not(.WdsButtonSplit--disabled) .WdsButtonSplit__dropdownTrigger[data-v-e9224293]:hover{background:var(--wdsColorBlue3)}.WdsButtonSplit--size-default .WdsButtonSplit__main[data-v-e9224293]{font-size:14px}.WdsButtonSplit--size-small .WdsButtonSplit__main[data-v-e9224293]{font-size:12px}}@layer wf{.BlueprintToolbarBlocksDropdown[data-v-e0d0a711]{min-width:185px;border:1px solid var(--wdsColorGray2);border:none;background:#fff;z-index:2;width:100%;max-height:40vh;border-radius:8px;padding:8px 10px;box-shadow:var(--wdsShadowMenu);box-sizing:border-box;color:var(--wdsColorGray6);max-height:300px;overflow-y:auto}.BlueprintToolbarBlocksDropdown__header[data-v-e0d0a711]{font-size:12px;text-transform:uppercase;font-weight:500;margin-bottom:10px}.BlueprintToolbarBlocksDropdown__list[data-v-e0d0a711]{display:flex;flex-direction:column;gap:8px}.BlueprintToolbarBlocksDropdown__list__item[data-v-e0d0a711]{display:grid;grid-template-columns:auto minmax(0,1fr) auto;gap:16px;align-items:center}.BlueprintToolbarBlocksDropdown__list__item__description[data-v-e0d0a711],.BlueprintToolbarBlocksDropdown__list__item__title[data-v-e0d0a711]{text-overflow:ellipsis;white-space:nowrap;overflow:hidden}.BlueprintToolbarBlocksDropdown__list__item__description[data-v-e0d0a711]{font-weight:400;color:var(--wdsColorGray4)}.BlueprintToolbarBlocksDropdown__list__item--disabled .BlueprintToolbarBlocksDropdown__list__item__description[data-v-e0d0a711],.BlueprintToolbarBlocksDropdown__list__item--disabled .BlueprintToolbarBlocksDropdown__list__item__title[data-v-e0d0a711]{opacity:50%}}@layer wf{.BlueprintToolbar__runBlueprint[data-v-5b7eb059]{min-width:160px}.BlueprintToolbar__runBlueprintDropdown[data-v-5b7eb059]{min-width:185px}}
