@layer wf{.SharedCopyClipboardButton[data-v-92570a7f]{position:relative}.menu[data-v-92570a7f]{width:160px}@keyframes pulse-92570a7f{0%{transform:scale(1)}50%{transform:scale(1.2)}to{transform:scale(1)}}.button.copied[data-v-92570a7f] svg{animation:pulse-92570a7f .3s ease-in-out}.button.copied[data-v-92570a7f]{color:#4caf50}}
