@layer wf{.WdsDropdownMenuItem[data-v-8fc4c139]{background-color:transparent;border:none;display:block;width:100%;display:grid;column-gap:8px;align-items:center;border-radius:4px;padding:8px;font-weight:400;font-size:.75rem;color:var(--wdsColorGray6);cursor:pointer;transition:all .2s;pointer-events:all}.WdsDropdownMenuItem[data-v-8fc4c139]:disabled{opacity:40%;cursor:not-allowed!important}.WdsDropdownMenuItem:has(.WdsDropdownMenuItem__icon) .WdsDropdownMenuItem__detail[data-v-8fc4c139]{grid-column-start:2}.WdsDropdownMenuItem__icon[data-v-8fc4c139]{grid-row-start:1;grid-row-end:-1;display:flex;align-items:center}.WdsDropdownMenuItem[data-v-8fc4c139]:hover{cursor:pointer;background-color:var(--wdsColorBlue1)}.WdsDropdownMenuItem--selected[data-v-8fc4c139]{background-color:var(--wdsColorBlue2)}.WdsDropdownMenuItem--danger[data-v-8fc4c139]{color:var(--wdsColorOrange5)}.WdsDropdownMenuItem__detail[data-v-8fc4c139],.WdsDropdownMenuItem__label[data-v-8fc4c139],.WdsDropdownMenuItem__label span[data-v-8fc4c139]{text-overflow:ellipsis;white-space:nowrap;overflow:hidden;text-align:left}.WdsDropdownMenuItem__label[data-v-8fc4c139]{display:flex;justify-content:space-between;gap:4px;width:100%}.WdsDropdownMenuItem__label__shortcut[data-v-8fc4c139]{color:var(--wdsColorGray4)}.WdsDropdownMenuItem[data-v-8fc4c139]:has(.WdsDropdownMenuItem__detail){grid-template-rows:auto auto}.WdsDropdownMenuItem__detail[data-v-8fc4c139]{grid-row:2;color:var(--wdsColorGray4)}.WdsDropdownMenuItem__icon--iconColor[data-v-8fc4c139]{display:flex;align-items:center;justify-content:center;height:18px;width:18px;border-radius:4px}.WdsDropdownMenuItem__action[data-v-8fc4c139]{grid-row:1 / -1;grid-column:-1;display:flex;align-items:center}}@layer wf{.WdsDropdownMenu[data-v-9826426e]{position:absolute;border:none;background:#fff;z-index:2;width:100%;max-height:40vh;overflow-y:auto;border-radius:8px;padding:8px 10px;box-shadow:var(--wdsShadowMenu);box-sizing:border-box}.WdsDropdownMenu[data-v-9826426e]:has(.WdsDropdownMenu__header){padding-top:0}.WdsDropdownMenu__checkbox[data-v-9826426e],.WdsDropdownMenu__item[data-v-9826426e]{min-height:36px}.WdsDropdownMenu__checkbox[data-v-9826426e]{margin-top:8px;margin-bottom:8px}.WdsDropdownMenu__item[data-v-9826426e]{background-color:transparent;border:none;display:block;width:100%;display:grid;grid-template-columns:1fr auto;column-gap:8px;align-items:center;border-radius:4px;padding:8px;font-weight:400;font-size:.75rem;color:var(--wdsColorGray6);cursor:pointer;transition:all .2s;pointer-events:all}.WdsDropdownMenu__itemLazyLoader[data-v-9826426e]{padding:12px}.WdsDropdownMenu__header[data-v-9826426e]{position:sticky;top:0;padding-top:8px;padding-bottom:8px;background:#fff;display:flex;flex-direction:column;gap:8px;z-index:2}.WdsDropdownMenu__header__search[data-v-9826426e]{background-color:var(--wdsColorGray1);border-radius:4px;height:36px;width:100%;display:flex;align-items:center;justify-content:center;gap:4px;padding:8px;border:1px solid transparent;color:currentcolor}.WdsDropdownMenu__header__search[data-v-9826426e]:hover{border-color:var(--wdsColorBlue1)}.WdsDropdownMenu__header__search[data-v-9826426e]:focus-within{background-color:#fff;outline:4px solid var(--wdsColorBlue1)}.WdsDropdownMenu__header__search__input[data-v-9826426e]{letter-spacing:inherit;color:currentcolor;padding:4px 0 5px;border:0px;box-sizing:content-box;background:none;height:1.4375em;margin:0;display:block;min-width:0px;width:100%;animation-name:mui-auto-fill-cancel;animation-duration:10ms;-webkit-appearance:textfield;-moz-appearance:textfield;appearance:textfield}.WdsDropdownMenu__header__search__input[data-v-9826426e]:focus-visible{outline:none}.WdsDropdownMenu__header__multiSelect[data-v-9826426e]{display:flex;gap:12px}.WdsDropdownMenu__header__multiSelect__count[data-v-9826426e],.WdsDropdownMenu__header__multiSelect__clear[data-v-9826426e]{text-transform:uppercase;font-size:13px}.WdsDropdownMenu__header__multiSelect__count[data-v-9826426e]{flex-grow:1;color:var(--wdsColorGray6)}.WdsDropdownMenu__header__multiSelect__clear[data-v-9826426e]{background-color:transparent;border:none;cursor:pointer;text-align:right;color:var(--wdsColorGray4)}.WdsDropdownMenu__header__search--no-results[data-v-9826426e],.WdsDropdownMenu__header__search--no-results[data-v-9826426e]:hover{border-color:var(--wdsColorOrange2, #e53e3e)!important}.WdsDropdownMenu__header__search--no-results[data-v-9826426e]:focus-within{border-color:var(--wdsColorOrange2, #e53e3e)!important;outline:4px solid rgba(229,62,62,.2)!important}.WdsDropdownMenu__no-results[data-v-9826426e]{padding:12px;text-align:center;color:var(--wdsColorGray5, #6b7280);font-size:.875rem}}
