import{aO as h}from"./index-dyx4v2bZ.js";function m(t,r){for(var a=0;a<r.length;a++){const o=r[a];if(typeof o!="string"&&!Array.isArray(o)){for(const e in o)if(e!=="default"&&!(e in t)){const i=Object.getOwnPropertyDescriptor(o,e);i&&Object.defineProperty(t,e,i.get?i:{enumerable:!0,get:()=>o[e]})}}}return Object.freeze(Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}))}const u=(t,{fastUrl:r,forceOverride:a}={})=>{if(a)chmln=void 0;else if(typeof chmln<"u"){console.log(`The global variable \`chmln\` is already defined.
Please make sure you're not adding Chameleon to the page other than with this library. Alternatively, you can use the forceOverride option to remove any previous declaration.

e.g.   chmln.init('token', { forceOverride: true });`);return}var o=document,e=window,i=t,n="chmln",l="identify alias track clear set show on off custom help _data".split(" "),s=o.createElement("script");if(e[n]||(e[n]={}),!e[n].root){e[n].accountToken=i,e[n].location=e.location.href.toString(),e[n].fastUrl=r||"https://fast.trychameleon.com/",e[n].now=new Date;for(var c=0;c<l.length;c++)(function(){var d=e[n][l[c]+"_a"]=[];e[n][l[c]]=function(){d.push(arguments)}})();s.src=e[n].fastUrl+"messo/"+i+"/messo.min.js",s.async=!0,o.head.appendChild(s)}},p=(t,r)=>{if(!chmln||!chmln.identify){console.log(`Failed to identify the user [${t}] because the chameleon.io script was not initialized yet.`);return}chmln.identify(t,r||{})},y=t=>{if(!chmln||!chmln.track){console.log(`Failed to track event [${t}] because the chameleon.io script was not initialized yet.`);return}chmln.track(t)};var f={init:u,identify:p,track:y};const g=h(f),b=m({__proto__:null,default:g},[f]);export{b as i};
