import{d as o,c as t,bk as n,o as l,f as r,g as e,K as i,_ as d}from"./index-dyx4v2bZ.js";const c={class:"BuilderInsertionOverlay"},_={class:"progressBar"},p=o({__name:"BuilderInsertionOverlay",setup(u){const a=t(()=>({"animation-duration":`${n}ms`}));return(m,s)=>(l(),r("div",c,[s[0]||(s[0]=e("div",{class:"overlay"},null,-1)),e("div",_,[e("div",{class:"elapsed",style:i(a.value)},null,4)])]))}}),f=d(p,[["__scopeId","data-v-0c332d4f"]]);export{f as default};
