@layer wf{.WdsCheckbox[data-v-56650334]{display:grid;grid-template-columns:auto 1fr;grid-template-rows:auto;align-items:center;column-gap:12px;row-gap:2px;cursor:pointer;text-align:center}.WdsCheckbox--disabled[data-v-56650334]{opacity:40%;cursor:not-allowed}.WdsCheckbox:has(input[aria-invalid=true]) .WdsCheckbox__checkbox[data-v-56650334]{border-color:var(--wdsColorOrange5)}.WdsCheckbox:has(input[aria-invalid=true]) .WdsCheckbox__label[data-v-56650334]{color:var(--wdsColorOrange5)}.WdsCheckbox__checkbox[data-v-56650334]{border:1px solid var(--wdsColorGray4);width:18px;height:18px;display:flex;align-items:center;justify-content:center;border-radius:4px;grid-row-start:1;grid-row-end:-1}.WdsCheckbox--checked .WdsCheckbox__checkbox[data-v-56650334]{background-color:var(--wdsColorBlue5);font-weight:700}.WdsCheckbox__checkbox[data-v-56650334] svg{display:none}.WdsCheckbox:hover:not(.WdsCheckbox--checked):not(.WdsCheckbox--disabled) .WdsCheckbox__checkbox[data-v-56650334] svg{display:block;color:var(--wdsColorGray4)}.WdsCheckbox--checked .WdsCheckbox__checkbox[data-v-56650334] svg{display:block;color:var(--wdsColorWhite)}.WdsCheckbox__label[data-v-56650334]{font-size:14px;text-overflow:ellipsis;white-space:nowrap;overflow:hidden}.WdsCheckbox__detail[data-v-56650334]{font-size:12px}.WdsCheckbox__label[data-v-56650334],.WdsCheckbox__detail[data-v-56650334]{line-height:180%;text-align:left}.WdsCheckbox[data-v-56650334]:has(.WdsCheckbox__detail){align-items:start;grid-template-rows:auto auto}.WdsCheckbox:has(.WdsCheckbox__detail) .WdsCheckbox__checkbox[data-v-56650334]{margin-top:14px}.WdsCheckbox__detail[data-v-56650334]{color:var(--wdsColorGray4)}.WdsCheckbox input[data-v-56650334]{display:none}}
