@layer wf{.sharedBlueprintNotice[data-v-f4d50b43]{display:flex;align-items:flex-start;gap:8px;padding:12px;margin-bottom:12px;background:var(--builderSubtleBackgroundColor, #f8fafc);border-radius:6px;font-size:12px;line-height:1.4;color:var(--builderSecondaryTextColor)}.sharedBlueprintNotice[data-v-f4d50b43] svg{flex-shrink:0;margin-top:1px}.category .header[data-v-f4d50b43]{display:flex;align-items:center;justify-content:space-between;font-size:12px;font-weight:500;line-height:12px;letter-spacing:1.3px;text-transform:uppercase;color:var(--builderSecondaryTextColor);margin-bottom:8px}.header__library-btn[data-v-f4d50b43]{display:flex;align-items:center;justify-content:center;width:20px;height:20px;padding:0;border:none;background:transparent;cursor:pointer;color:var(--builderSecondaryTextColor);border-radius:4px;opacity:.7;transition:opacity .2s,background .2s}.header__library-btn[data-v-f4d50b43]:hover{opacity:1;background:var(--builderSubtleSeparatorColor)}.header__library-btn[data-v-f4d50b43] svg{width:14px;height:14px}.tools[data-v-f4d50b43]{display:grid;grid-template-columns:1fr;grid-template-rows:auto}.tool[data-v-f4d50b43]{display:grid;grid-template-columns:18px 1fr auto;grid-template-rows:1fr;column-gap:8px;padding:8px;border-radius:4px;cursor:grab;position:relative}.tool--shared[data-v-f4d50b43]{grid-template-columns:18px 1fr auto}.tool img[data-v-f4d50b43]{max-width:18px;max-height:18px;aspect-ratio:1 / 1}.tool[data-v-f4d50b43]:hover{background:var(--builderSubtleSeparatorColor)}.tool__delete[data-v-f4d50b43]{display:none;align-items:center;justify-content:center;width:20px;height:20px;padding:0;border:none;background:transparent;cursor:pointer;color:var(--builderSecondaryTextColor);border-radius:4px;opacity:.6;transition:opacity .2s,background .2s}.tool--shared:hover .tool__delete[data-v-f4d50b43]{display:flex}.tool__delete[data-v-f4d50b43]:hover{opacity:1;background:var(--builderSubtleSeparatorColor);color:var(--builderErrorColor)}.tool__delete[data-v-f4d50b43]:active{opacity:.8}.BuilderSidebarPanel__footer__actions[data-v-f4d50b43]{flex:0 0 var(--builderPanelSwitcherHeight);bottom:0;height:var(--builderPanelSwitcherHeight);display:flex;flex-direction:column;align-items:center;justify-content:center;gap:8px;border-top:1px solid var(--builderSeparatorColor);background:var(--builderBackgroundColor);padding:8px}.BuilderSidebarPanel__footer__btn[data-v-f4d50b43]{font-size:12px;font-weight:500;line-height:180%;min-width:190px}}
