import{r as t,aD as c,Y as v,Z as r}from"./index-dyx4v2bZ.js";function f(l,o){const e=t(!1),s=t(),u=c(`useListResources(${o})`,Array.isArray),a=v(u.value??[]);async function n(){e.value=!0,s.value=void 0;try{a.value=await l.sendListResourcesRequest(o),u.value=a.value}catch(i){s.value=i,a.value=[]}finally{e.value=!1}}return{data:r(a),isLoading:r(e),error:r(s),load:n}}export{f as u};
