import{d as n,c as u,o as s,f as d,J as v,S as b,G as r,_ as o,an as m,ap as p,D as T,E as W,b as f,z as y,A as k,t as g}from"./index-dyx4v2bZ.js";const h=["disabled","data-writer-tooltip"],C={key:0,class:"WdsTab__indicator"},S=n({__name:"WdsTab",props:{disabled:{type:[Boolean,String],default:void 0},selected:{type:Boolean},variant:{type:String,required:!0}},emits:{click:()=>!0},setup(e){const t=e,l=u(()=>typeof t.disabled=="string"?t.disabled:void 0);return(i,a)=>(s(),d("button",{class:r(["WdsTab",{"WdsTab--disabled":e.disabled,"WdsTab--selected":e.selected,"WdsTab--variant-bar":e.variant==="bar","WdsTab--variant-chip":e.variant==="chip"}]),type:"button",disabled:!!e.disabled,"data-writer-tooltip":l.value,onClick:a[0]||(a[0]=c=>i.$emit("click"))},[v(i.$slots,"default",{},void 0,!0),e.variant==="bar"?(s(),d("div",C)):b("",!0)],10,h))}}),B=o(S,[["__scopeId","data-v-db73c62c"]]),V=n({__name:"WdsTabs",props:m({tabs:{type:Array,required:!0},variant:{type:String,default:"chip"}},{modelValue:{type:String},modelModifiers:{}}),emits:["update:modelValue"],setup(e){const t=p(e,"modelValue");return(l,i)=>(s(),d("div",{class:r(["WdsTabs",{"WdsTabs--variant-bar":e.variant==="bar","WdsTabs--variant-chip":e.variant==="chip"}])},[(s(!0),d(T,null,W(e.tabs,a=>(s(),f(B,{key:a.value,value:a.value,variant:e.variant,disabled:a.disabled,selected:a.value===t.value,onClick:c=>t.value=a.value},{default:y(()=>[k(g(a.label),1)]),_:2},1032,["value","variant","disabled","selected","onClick"]))),128))],2))}}),x=o(V,[["__scopeId","data-v-6e6e5572"]]);export{x as W};
